@@ -2,7 +2,10 @@ package e2e
 
 import (
 	"flag"
+	"fmt"
 	"os"
+	"path/filepath"
+	"strings"
 	"testing"
 
 	"github.com/sirupsen/logrus"
@@ -17,11 +20,10 @@ import (
 var (
 	df *deployframework.DeployFramework
 
-	kubeConfig         string
-	logLevel           string
-	runTestsLocal      bool
-	runDevSetup        bool
-	runAllInstallTests bool
+	kubeConfig    string
+	logLevel      string
+	runTestsLocal bool
+	runDevSetup   bool
 
 	meteringOperatorImageRepo  string
 	meteringOperatorImageTag   string
@@ -41,14 +43,20 @@ var (
 	catalogSourceName              string
 	catalogSourceNamespace         string
 
-	kubeNamespaceCharLimit          = 63
-	namespacePrefixCharLimit        = 10
-	packageName                     = "metering-ocp"
-	preUpgradeTestDirName           = "pre-upgrade"
-	postUpgradeTestDirName          = "post-upgrade"
-	gatherTestArtifactsScript       = "gather-test-install-artifacts.sh"
+	kubeNamespaceCharLimit   = 63
+	namespacePrefixCharLimit = 10
+
+	packageName            = "metering-ocp"
+	preUpgradeTestDirName  = "pre-upgrade"
+	postUpgradeTestDirName = "post-upgrade"
+
+	hackScriptDirName               = "hack"
+	registryArtifactsDirName        = "registry"
 	testMeteringConfigManifestsPath = "/test/e2e/manifests/meteringconfigs/"
 	testNFSManifestPath             = "/test/e2e/manifests/nfs/"
+
+	gatherTestArtifactsScript         = "gather-test-install-artifacts.sh"
+	registryGatherTestArtifactsScript = "gather-registry-artifacts.sh"
 )
 
 func init() {
@@ -81,7 +89,6 @@ func testMainWrapper(m *testing.M) int {
 
 	flag.BoolVar(&runTestsLocal, "run-tests-local", false, "Controls whether the metering and reporting operators are run locally during tests")
 	flag.BoolVar(&runDevSetup, "run-dev-setup", false, "Controls whether the e2e suite uses the dev-friendly configuration")
-	flag.BoolVar(&runAllInstallTests, "test-run-all-install-tests", true, "Controls whether or not the e2e suite installs all of the tests listed in the TestManualMeteringInstall function")
 	flag.BoolVar(&runAWSBillingTests, "run-aws-billing-tests", runAWSBillingTests, "")
 
 	flag.StringVar(&meteringOperatorImageRepo, "metering-operator-image-repo", meteringOperatorImageRepo, "")
@@ -121,15 +128,37 @@ func testMainWrapper(m *testing.M) int {
 	var registryProvisioned bool
 	catalogSourceName, catalogSourceNamespace, err = df.CreateCatalogSourceFromIndex(indexImage)
 	if err != nil {
-		df.Logger.Fatalf("Failed to create the CatalogSource custom resource using the %s index image: %v", indexImage, err)
+		df.Logger.Errorf("Failed to create the CatalogSource custom resource using the %s index image: %v", indexImage, err)
+		return 1
 	}
 	if !df.RunDevSetup {
-		defer df.DeleteRegistryResources(registryProvisioned, catalogSourceName, catalogSourceNamespace)
+		var (
+			errors []string
+			err    error
+		)
+		defer func() {
+			resourceOutputPath := filepath.Join(testOutputPath, registryArtifactsDirName)
+			registryArtifactsGatherScript := filepath.Join(repoPath, hackScriptDirName, registryGatherTestArtifactsScript)
+
+			err = testhelpers.GatherRegistryResources(resourceOutputPath, registryArtifactsGatherScript, catalogSourceName, catalogSourceNamespace)
+			if err != nil {
+				errors = append(errors, fmt.Sprintf("failed to gather the OLM registry resources: %v", err))
+			}
+			err = df.DeleteRegistryResources(registryProvisioned, catalogSourceName, catalogSourceNamespace)
+			if err != nil {
+				errors = append(errors, fmt.Sprintf("failed to delete the registry resources: %v", err))
+			}
+		}()
+		if len(errors) != 0 {
+			df.Logger.Errorf(strings.Join(errors, "\n"))
+			return 1
+		}
 	}
 
 	err = df.WaitForPackageManifest(catalogSourceName, catalogSourceNamespace, subscriptionChannel)
 	if err != nil {
-		df.Logger.Fatalf("Failed to wait for the metering-ocp packagemanifest to become ready: %v", err)
+		df.Logger.Errorf("Failed to wait for the metering-ocp packagemanifest to become ready: %v", err)
+		return 1
 	}
 
 	return m.Run()
@@ -155,31 +184,12 @@ func TestManualMeteringInstall(t *testing.T) {
 		PreInstallFunc                 PreInstallFunc
 		MeteringConfigManifestFilename string
 	}{
-		{
-			Name:                      "HDFS-MissingStorageSpec",
-			MeteringOperatorImageRepo: meteringOperatorImageRepo,
-			MeteringOperatorImageTag:  meteringOperatorImageTag,
-			Skip:                      false,
-			ExpectInstallErr:          true,
-			ExpectInstallErrMsg: []string{
-				"failed to install metering",
-				"failed to create the MeteringConfig resource",
-				"spec.storage in body is required|spec.storage: Required value",
-			},
-			InstallSubTests: []InstallTestCase{
-				{
-					Name:     "testInvalidMeteringConfigMissingStorageSpec",
-					TestFunc: testInvalidMeteringConfigMissingStorageSpec,
-				},
-			},
-			MeteringConfigManifestFilename: "missing-storage.yaml",
-		},
 		{
 			Name:                      "HDFS-ValidNodeSelector",
 			MeteringOperatorImageRepo: meteringOperatorImageRepo,
 			MeteringOperatorImageTag:  meteringOperatorImageTag,
 			// TODO: transistion this to a periodic test and
-			// update the `Skip` condition to !runAllInstallTests
+			// update the `Skip` condition to !testing.Short()
 			// TODO: disabling this test for the time being as
 			// we're labeling nodes and firing off this metering
 			// installation before the machineautoscaler has provisioned
@@ -220,10 +230,10 @@ func TestManualMeteringInstall(t *testing.T) {
 			MeteringConfigManifestFilename: "node-selector-prometheus-importer-disabled.yaml",
 		},
 		{
-			Name:                      "HDFS-ReportDynamicInputData",
+			Name:                      "HDFS-MYSQL-ReportDynamicInputData",
 			MeteringOperatorImageRepo: meteringOperatorImageRepo,
 			MeteringOperatorImageTag:  meteringOperatorImageTag,
-			Skip:                      false,
+			PreInstallFunc:            createMySQLDatabase,
 			InstallSubTests: []InstallTestCase{
 				{
 					Name:     "testReportingProducesData",
@@ -240,8 +250,12 @@ func TestManualMeteringInstall(t *testing.T) {
 					Name:     "testFailedPrometheusQueryEvents",
 					TestFunc: testFailedPrometheusQueryEvents,
 				},
+				{
+					Name:     "testEnsurePostgresParametersAreMissing",
+					TestFunc: testEnsurePostgresParametersAreMissing,
+				},
 			},
-			MeteringConfigManifestFilename: "prometheus-metrics-importer-enabled.yaml",
+			MeteringConfigManifestFilename: "mysql.yaml",
 		},
 		{
 			Name:                      "HDFS-ReportStaticInputData",
@@ -268,53 +282,29 @@ func TestManualMeteringInstall(t *testing.T) {
 					Name:     "testLeaderElectionEventIsCreated",
 					TestFunc: testLeaderElectionEventIsCreated,
 				},
+				{
+					Name:     "testLeaderElectionConfigMapIsCreated",
+					TestFunc: testLeaderElectionConfigMapIsCreated,
+				},
 				{
 					Name:     "testFailedPrometheusQueryEvents",
 					TestFunc: testFailedPrometheusQueryEvents,
 				},
 				{
-					Name:         "testReportIsDeletedWhenNoDeps",
-					TestFunc:     testReportIsDeletedWhenNoDeps,
-					ExtraEnvVars: []string{},
+					Name:     "testReportIsDeletedWhenNoDeps",
+					TestFunc: testReportIsDeletedWhenNoDeps,
 				},
 				{
-					Name:         "testReportIsNotDeletedWhenReportDependsOnIt",
-					TestFunc:     testReportIsNotDeletedWhenReportDependsOnIt,
-					ExtraEnvVars: []string{},
+					Name:     "testReportIsNotDeletedWhenReportDependsOnIt",
+					TestFunc: testReportIsNotDeletedWhenReportDependsOnIt,
+				},
+				{
+					Name:     "testEnsurePostgresParametersAreMissing",
+					TestFunc: testEnsurePostgresParametersAreMissing,
 				},
 			},
 			MeteringConfigManifestFilename: "prometheus-metrics-importer-disabled.yaml",
 		},
-		{
-			Name:                      "HDFS-MySQLDatabase",
-			MeteringOperatorImageRepo: meteringOperatorImageRepo,
-			MeteringOperatorImageTag:  meteringOperatorImageTag,
-			// TODO: disable this for now as the mysql:5.7 image
-			// stream was recently removed from 4.6 and there are
-			// some issues with using mariadb as a direct replacement
-			// as Hive server hangs during the create table call
-			// and requires a restart to work properly.
-			Skip:           true,
-			PreInstallFunc: createMySQLDatabase,
-			InstallSubTests: []InstallTestCase{
-				{
-					Name:     "testReportingProducesData",
-					TestFunc: testReportingProducesData,
-					ExtraEnvVars: []string{
-						"REPORTING_OPERATOR_PROMETHEUS_DATASOURCE_MAX_IMPORT_BACKFILL_DURATION=15m",
-						"REPORTING_OPERATOR_PROMETHEUS_METRICS_IMPORTER_INTERVAL=30s",
-						"REPORTING_OPERATOR_PROMETHEUS_METRICS_IMPORTER_CHUNK_SIZE=5m",
-						"REPORTING_OPERATOR_PROMETHEUS_METRICS_IMPORTER_INTERVAL=5m",
-						"REPORTING_OPERATOR_PROMETHEUS_METRICS_IMPORTER_STEP_SIZE=60s",
-					},
-				},
-				{
-					Name:     "testFailedPrometheusQueryEvents",
-					TestFunc: testFailedPrometheusQueryEvents,
-				},
-			},
-			MeteringConfigManifestFilename: "mysql.yaml",
-		},
 		{
 			Name:                      "S3-ReportDynamicInputData",
 			MeteringOperatorImageRepo: meteringOperatorImageRepo,
@@ -339,9 +329,8 @@ func TestManualMeteringInstall(t *testing.T) {
 					},
 				},
 				{
-					Name:         "testEnsureS3BucketIsDeleted",
-					TestFunc:     testEnsureS3BucketIsDeleted,
-					ExtraEnvVars: []string{},
+					Name:     "testEnsureS3BucketIsDeleted",
+					TestFunc: testEnsureS3BucketIsDeleted,
 				},
 				{
 					Name:     "testFailedPrometheusQueryEvents",
@@ -354,7 +343,7 @@ func TestManualMeteringInstall(t *testing.T) {
 			Name:                      "NFS-ReportDynamicInputData",
 			MeteringOperatorImageRepo: meteringOperatorImageRepo,
 			MeteringOperatorImageTag:  meteringOperatorImageTag,
-			Skip:                      !runAllInstallTests,
+			Skip:                      !testing.Short(),
 			PreInstallFunc:            createNFSProvisioner,
 			InstallSubTests: []InstallTestCase{
 				{
